#version 330

#define WIDTH 1800.0
#define HEIGHT 1000.0

#define SENSITIVITY 5.0

in vec2 textPos;

uniform sampler2D depthTexture;

layout(location = 0) out vec4 fragColor;


// checks if the value is between the tresholds
float threshold (in float lowerThreshold, in float upperThreshold , in float value) {

    if (value < lowerThreshold) {
        return 0.0;
    }

	if (value > upperThreshold) {
        return 1.0;
    }

	return value;

}


// averages the pixel RGB-intensities
float avgIntensity (in vec4 pix) {

    return (pix.r + pix.g + pix.b) / 3.0;

}


// returns the color of a pixel in the given texture
vec4 getPixel (in vec2 coords, in float dx, in float dy) {

    vec4 color = texture2D(depthTexture, coords + vec2(dx, dy));

	// power the color-values, since they are pretty close to 1
	color = vec4(pow(color.x, 200), pow(color.y, 200), pow(color.z, 200), 1.0);

	return color;

}


// returns the pixel color
float checkForEdge (in vec2 coords) {

    float dx = 1.0 / WIDTH;
    float dy = 1.0 / HEIGHT;

    float pix[9];
    int k = -1;
    float delta;

    // read neighboring pixel intensities
	for (int i = -1; i < 2; i++) {
        for(int j = -1; j < 2; j++) {
            k++;
            pix[k] = avgIntensity(getPixel(coords, float(i) * dx, float(j) * dy));
		}
	}

	// average color differences around neighboring pixels
	delta = (abs(pix[1] - pix[7]) + abs(pix[5] - pix[3]) + abs(pix[0] - pix[8]) + abs(pix[2] - pix[6])) * SENSITIVITY;
    return threshold(0.4, 0.5, clamp(1.8 * delta, 0.0, 1.0));

}


// main-method
void main() {
	
	// check if the pixel is at the edge of the texture
	if (textPos.x < 0.001 || textPos.x > 0.999 || textPos.y < 0.001 || textPos.y > 0.999) {
		
		fragColor = vec4(0.0, 0.0, 0.0, 1.0);

	} else {
	
		fragColor = vec4(0.0, 0.0, 0.0, 1.0);
		float edgeColor = checkForEdge(textPos);

		fragColor.r = edgeColor;
		fragColor.g = edgeColor;
		fragColor.b = edgeColor;

	}

}